#apihelper.py - API class for making network calls

from urllib import parse
import requests
import logging
import json
import time
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from veracode_api_signing.exceptions import VeracodeAPISigningException
from veracode_api_signing.plugin_requests import RequestsAuthPluginVeracodeHMAC

from .exceptions import VeracodeAPIError
from .log import VeracodeLog as vlog

logger = logging.getLogger(__name__)

class APIHelper():
    def __init__(self, proxies=None, debug=False):
        self.baseurl = "https://analysiscenter.veracode.com/api"
        requests.Session().mount(self.baseurl, HTTPAdapter(max_retries=3))
        self.proxies = proxies
        self.base_rest_url = "https://api.veracode.com/"
        self.retry_seconds = 120
        self.connect_error_msg = "Connection Error"
        # vlog.setup_logging(self,debug=debug)

    # helper functions

    def _rest_request(self, url, method, params=None,body=None,fullresponse=False,use_base_url=True):
        # base request method for a REST request
        myheaders = {"User-Agent": "api.py"}
        if method in ["POST", "PUT"]:
            myheaders.update({'Content-type': 'application/json'})

        retry_strategy = Retry(total=3,
            status_forcelist=[429, 500, 502, 503, 504],
            method_whitelist=["HEAD", "GET", "OPTIONS"]
            )
        session = requests.Session()
        session.mount(self.base_rest_url, HTTPAdapter(max_retries=retry_strategy))

        if use_base_url:
            url = self.base_rest_url + url

        try:
            if method == "GET":
                request = requests.Request(method, url, params=params, auth=RequestsAuthPluginVeracodeHMAC(), headers=myheaders)
                prepared_request = request.prepare()
                r = session.send(prepared_request, proxies=self.proxies)
            elif method == "POST":
                r = requests.post(url, params=params,auth=RequestsAuthPluginVeracodeHMAC(),headers=myheaders,data=body)
            elif method == "PUT":
                r = requests.put(url, params=params,auth=RequestsAuthPluginVeracodeHMAC(), headers=myheaders,data=body)
            elif method == "DELETE":
                r = requests.delete(url, params=params,auth=RequestsAuthPluginVeracodeHMAC(),headers=myheaders)
            else:
                raise VeracodeAPIError("Unsupported HTTP method")
        except requests.exceptions.RequestException as e:
            logger.exception(self.connect_error_msg)
            raise VeracodeAPIError(e)

        if not (r.status_code == requests.codes.ok):
            logger.debug("API call returned non-200 HTTP status code: {}".format(r.status_code))

        if not (r.ok):
            logger.debug("Error retrieving data. HTTP status code: {}".format(r.status_code))
            if r.status_code == 401:
                logger.exception("Error [{}]: {} for request {}. Check that your Veracode API account credentials are correct.".format(r.status_code,
                                r.text, r.request.url))
            else:
                logger.exception("Error [{}]: {} for request {}".
                    format(r.status_code, r.text, r.request.url))
            raise requests.exceptions.RequestException()

        if fullresponse:
            return r
        elif r.text != "":
            return r.json()
        else:
            return ""

    def _rest_paged_request(self, uri, method, element, params=None):
        all_data = []
        page = 0
        more_pages = True

        while more_pages:
            params['page']=page
            page_data = self._rest_request(uri,method,params)
            total_pages = page_data.get('page', {}).get('total_pages', 0)
            data_page = page_data.get('_embedded', {}).get(element, [])
            all_data += data_page  
            
            page += 1
            more_pages = page < total_pages
        return all_data

    def _xml_request(self, url, method, params=None):
        # base request method for XML APIs, handles what little error handling there is around these APIs
        if method not in ["GET", "POST"]:
            raise VeracodeAPIError("Unsupported HTTP method")

        try:
            session = requests.Session()
            session.mount(self.baseurl, HTTPAdapter(max_retries=3))
            request = requests.Request(method, url, params=params, auth=RequestsAuthPluginVeracodeHMAC(),headers={"User-Agent": "api.py"})
            prepared_request = request.prepare()
            r = session.send(prepared_request, proxies=self.proxies)
            if 200 <= r.status_code <= 299:
                if r.status_code == 204:
                    #retry after wait
                    time.sleep(self.retry_seconds)
                    return self._request(url,method,params)
                elif r.content is None:
                    logger.debug("HTTP response body empty:\r\n{}\r\n{}\r\n{}\r\n\r\n{}\r\n{}\r\n{}\r\n"
                                  .format(r.request.url, r.request.headers, r.request.body, r.status_code, r.headers, r.content))
                    raise VeracodeAPIError("HTTP response body is empty")
                else:
                    return r.content
            else:
                logger.debug("HTTP error for request:\r\n{}\r\n{}\r\n{}\r\n\r\n{}\r\n{}\r\n{}\r\n"
                              .format(r.request.url, r.request.headers, r.request.body, r.status_code, r.headers, r.content))
                raise VeracodeAPIError("HTTP error: {}".format(r.status_code))
        except requests.exceptions.RequestException as e:
            logger.exception("Connection error")
            raise VeracodeAPIError(e)


# Purpose:  API utilities
#
# Notes:    API credentials must be enabled on Veracode account and placed in ~/.veracode/credentials like
#
#           [default]
#           veracode_api_key_id = <YOUR_API_KEY_ID>
#           veracode_api_key_secret = <YOUR_API_KEY_SECRET>
#
#           and file permission set appropriately (chmod 600)

import requests
import logging
from requests.adapters import HTTPAdapter
from typing import List

from veracode_api_signing.exceptions import VeracodeAPISigningException
from veracode_api_signing.plugin_requests import RequestsAuthPluginVeracodeHMAC

from .constants import Constants
from .exceptions import VeracodeAPIError
from .applications import Applications, Sandboxes, CustomFields
from .findings import Findings, SummaryReport
from .policy import Policies
from .sca import Workspaces
from .collections import Collections
from .identity import Users, Teams, BusinessUnits, APICredentials, Roles
from .healthcheck import Healthcheck
from .dynamic import Analyses, Scans, Occurrences, Configuration, CodeGroups, ScanCapacitySummary, ScanOccurrences, ScannerVariables, DynUtils
from .xmlapi import XMLAPI

class VeracodeAPI:

    def __init__(self, proxies=None):
        self.baseurl = 'https://analysiscenter.veracode.com/api'
        requests.Session().mount(self.baseurl, HTTPAdapter(max_retries=3))
        self.proxies = proxies
        self.retry_seconds = 120
        self.connect_error_msg = "Connection Error"

    #xml apis

    def get_app_list(self):
        return XMLAPI().get_app_list()

    def get_app_info(self, app_id):
        return XMLAPI().get_app_info(app_id)

    def get_sandbox_list(self, app_id):
        return XMLAPI().get_sandbox_list(app_id)

    def get_build_list(self, app_id, sandbox_id=None):
        return XMLAPI().get_build_list(app_id, sandbox_id)
    
    def get_build_info(self, app_id, build_id=None, sandbox_id=None):
        return XMLAPI().get_build_info(app_id,build_id,sandbox_id)

    def get_detailed_report(self, build_id):
        return XMLAPI().get_detailed_report(build_id)

    def set_mitigation_info(self,build_id,flaw_id_list,action,comment):
        return XMLAPI().set_mitigation_info(build_id,flaw_id_list,action,comment)

    def generate_archer(self,payload):
        return XMLAPI().generate_archer(payload)

    def download_archer(self, token=None):
        return XMLAPI().download_archer(token)

    # rest apis

    ## Healthcheck APIs

    def healthcheck(self):
        return Healthcheck().healthcheck()

    def status(self):
        return Healthcheck().status()

    ## Application and Sandbox APIs

    def get_apps(self):
        return Applications().get_all()

    def get_app (self,guid=None,legacy_id=None):
        return Applications().get(guid,legacy_id)

    def get_app_by_name (self,appname):
        return Applications().get_by_name(appname)

    def create_app(self,app_name,business_criticality, business_unit=None, teams=[]):
        return Applications().create(app_name,business_criticality,business_unit,teams)

    def delete_app (self,guid):
        return Applications().delete(guid)

    def get_custom_fields (self):
        return CustomFields().get_all()

    def get_app_sandboxes (self,guid):
        return Sandboxes().get_all(guid)

    def create_sandbox (self, app, name, auto_recreate=False, custom_fields=[]):
        return Sandboxes().create(app,name,auto_recreate,custom_fields)

    def update_sandbox (self, app, sandbox, name, auto_recreate=False, custom_fields=[]):
        return Sandboxes().update(app,sandbox,name,auto_recreate,custom_fields)

    def delete_sandbox (self, app, sandbox):
        return Sandboxes().delete(app,sandbox)

    # Policy APIs

    def get_policies (self):
        return Policies().get_all()

    def get_policy (self,guid):
        return Policies().get(guid)

    def create_policy(self, name, description, vendor_policy=False, finding_rules=[], scan_frequency_rules=[], grace_periods={}):
        return Policies().create(name, description, vendor_policy, finding_rules, scan_frequency_rules, grace_periods)

    def delete_policy (self,guid):
        return Policies().delete(guid)

    def update_policy(self, guid, name, description, vendor_policy=False, finding_rules=[], scan_frequency_rules=[], grace_periods={}):
        return Policies().update(guid, name, description, vendor_policy, finding_rules, scan_frequency_rules, grace_periods)

    # Findings and Reporting APIs

    def get_findings(self,app,scantype='STATIC',annot='TRUE',request_params=None,sandbox=None):
        return Findings().get_findings(app,scantype,annot,request_params,sandbox)

    def get_static_flaw_info(self,app,issueid,sandbox=None):
        return Findings().get_static_flaw_info(app,issueid,sandbox)

    def get_dynamic_flaw_info(self,app,issueid):
        return Findings().get_dynamic_flaw_info(app,issueid)

    def get_summary_report(self,app,sandbox=None):
        return SummaryReport().get_summary_report(app,sandbox)

    def add_annotation(self,app,issue_list,comment,action,sandbox=None):
        return Findings().add_annotation(app,issue_list,comment,action,sandbox)

    ## Collections APIs

    def get_collections(self):
        return Collections().get_all()

    def get_collections_by_name(self,collection_name):
        return Collections().get_by_name(collection_name)

    def get_collections_by_business_unit(self,business_unit_name):
        return Collections().get_by_business_unit(business_unit_name)

    def get_collections_statistics(self):
        return Collections().get_statistics()

    def get_collection(self,guid):
        return Collections().get(guid)

    def get_collection_assets(self,guid):
        return Collections().get_assets(guid)

    def create_collection(self,name,description="",tags='',business_unit_guid=None,custom_fields=[],assets=[]):
        return Collections().create(name,description,tags,business_unit_guid,custom_fields,assets)

    def update_collection(self,guid,name,description="",tags="",business_unit_guid=None,custom_fields=[],assets=[]):
        return Collections().update(name,description,tags,business_unit_guid,custom_fields,assets)

    def delete_collection(self,guid):
        return Collections().delete(guid)

    ## Identity APIs

    def get_users(self):
        return Users().get_all()

    def get_user_self (self):
        return Users().get_self()

    def get_user(self,user_guid):
        return Users().get(user_guid)

    def get_user_by_name(self,username):
        return Users().get_by_name(username)

    def create_user (self,email,firstname,lastname,username=None,type="HUMAN",roles=[],teams=[]):
        return Users().create(email,firstname,lastname,username,type,roles,teams)

    def update_user_roles (self,user_guid,roles):
        return Users().update_roles(user_guid,roles)

    def update_user (self,user_guid,changes):
        return Users().update(user_guid,changes)
    
    def update_user_email_address (self,user_guid,email_address,ignore_verification=False):
        return Users().update_email_address(user_guid,email_address,ignore_verification)
    
    def send_password_reset (self,user_legacy_id):
        return Users().reset_password(user_legacy_id)

    def disable_user (self,user_guid):
        return Users().disable(user_guid)

    def delete_user (self,user_guid):
        return Users().delete(user_guid)

    def get_teams (self, all_for_org=False):
        return Teams().get_all()

    def create_team (self, team_name, business_unit=None, members=[]):        
        return Teams().create(team_name,business_unit,members)

    def update_team (self, team_guid, team_name="", business_unit=None, members=[]):
        return Teams().update(team_guid,team_name,business_unit,members)

    def delete_team (self, team_guid):
        return Teams().delete(team_guid)
            
    def get_business_units (self):
        return BusinessUnits().get_all()

    def get_business_unit (self, guid):
        return BusinessUnits().get(guid)

    def create_business_unit (self, name, teams=[]):
        return BusinessUnits().create(name,teams)

    def update_business_unit (self, guid, name='', teams=[]):
        return BusinessUnits().update(guid,name,teams)

    def delete_business_unit (self, guid):
        return BusinessUnits().delete(guid)

    def get_creds (self,api_id=None):
        if api_id != None:
            return APICredentials().get(api_id)
        else:
            return APICredentials().get_self()

    def renew_creds (self):
        return APICredentials().renew()

    def revoke_creds (self, api_id):
        return APICredentials().revoke(api_id)

    def get_roles (self):
        return Roles().get_all()

## SCA APIs - note must be human user to use these, not API user

    def get_workspaces(self):
        return Workspaces().get_all()

    def get_workspace_by_name(self,name):
        return Workspaces().get_by_name(name)

    def create_workspace(self,name):
        return Workspaces().create(name)

    def add_workspace_team(self,workspace_guid,team_id):
        return Workspaces().add_team(workspace_guid,team_id)

    def delete_workspace(self,workspace_guid):
        return Workspaces().delete(workspace_guid)

    def get_projects(self,workspace_guid):
        return Workspaces().get_projects(workspace_guid)

    def get_project(self,workspace_guid,project_guid):
        return Workspaces().get_project(workspace_guid,project_guid)

    def get_agents(self,workspace_guid):
        return Workspaces().get_agents(workspace_guid)

    def get_agent(self,workspace_guid,agent_guid):
        return Workspaces().get_agent(workspace_guid,agent_guid)

    def create_agent(self,workspace_guid,name,agent_type='CLI'):
        return Workspaces().create_agent(workspace_guid,name,agent_type)

    def get_agent_tokens(self,workspace_guid,agent_guid):
        return Workspaces().get_agent_tokens(workspace_guid,agent_guid)

    def get_agent_token(self,workspace_guid,agent_guid,token_id):
        return Workspaces().get_agent_token(workspace_guid,agent_guid,token_id)
        
    def regenerate_agent_token(self,workspace_guid,agent_guid):
        return Workspaces().regenerate_agent_token(workspace_guid,agent_guid)

    def revoke_agent_token(self,workspace_guid,agent_guid,token_id):
        return Workspaces().revoke_agent_token(workspace_guid,agent_guid,token_id)

    def get_issues(self,workspace_guid):
        return Workspaces().get_issues(workspace_guid)

    def get_issue(self,issue_id):
        return Workspaces().get_issues(issue_id)

    def get_libraries(self,workspace_guid,unmatched=False):
        return Workspaces().get_libraries(workspace_guid, unmatched)

    def get_library(self,library_id):
        return Workspaces().get_library(library_id)

    def get_vulnerability(self,vulnerability_id):
        return Workspaces().get_vulnerability(vulnerability_id)

    def get_license(self,license_id):
        return Workspaces().get_license(license_id)

    def get_sca_events(self,date_gte=None,event_group=None,event_type=None):
        return Workspaces().get_events(date_gte,event_group,event_type)

    def get_sca_scan(self,scan_id):
        return Workspaces().get_scan(scan_id)

    #dynamic APIs

    def get_analyses(self):
        return Analyses().get_all()

    def get_analyses_by_name(self,name):
        return Analyses().get_by_name(analysis_name=name)

    def get_analyses_by_target_url(self,url):
        return Analyses().get_by_target_url(target_url=url)

    def get_analyses_by_search_term(self,search_term):
        return Analyses().get_by_search_term(search_term=search_term)

    def get_analysis(self,analysis_id):
        return Analyses().get(guid=analysis_id)
    
    def get_analysis_audits(self,analysis_id):
        return Analyses().get_audits(guid=analysis_id)

    def get_analysis_scans(self,analysis_id):
        return Analyses().get_scans(guid=analysis_id)

    def get_analysis_scanner_variables(self,analysis_id):
        return Analyses().get_scanner_variables(guid=analysis_id)

    def create_analysis(self,name,scans,schedule_frequency='ONCE',business_unit_guid=None,email=None,owner=None):
        return Analyses().create(name,scans,schedule_frequency,business_unit_guid,email,owner)

    def update_analysis(self,guid,name,scans,schedule_frequency='ONCE',business_unit_guid=None,email=None,owner=None):
        return Analyses().update(guid,name,scans,schedule_frequency,business_unit_guid,email,owner)
    
    def update_analysis_scanner_variable(self,analysis_guid,scanner_variable_guid,reference_key,value,description):
        return Analyses().update_scanner_variable(analysis_guid,scanner_variable_guid,reference_key,value,description)

    def delete_analysis_scanner_variable(self,analysis_guid,scanner_variable_guid):
        return Analyses().delete_scanner_variable(analysis_guid,scanner_variable_guid)

    def delete_analysis(self,analysis_guid):
        return Analyses().delete(guid=analysis_guid)
    
    def get_dyn_scan(self,scan_guid):
        return Scans().get(guid=scan_guid)

    def get_dyn_scan_audits(self,scan_guid):
        return Scans().get_audits(guid=scan_guid)

    def get_dyn_scan_config(self,scan_guid):
        return Scans().get_configuration(guid=scan_guid)

    def update_dyn_scan(self,scan_guid,scan):
        return Scans().update(guid=scan_guid,scan=scan)

    def delete_dyn_scan(self,scan_guid):
        return Scans().delete(guid=scan_guid)

    def get_scan_scanner_variables(self,scan_id):
        return Scans().get_scanner_variables(guid=scan_id)

    def update_scan_scanner_variable(self,scan_guid,scanner_variable_guid,reference_key,value,description):
        return Scans().update_scanner_variable(scan_guid,scanner_variable_guid,reference_key,value,description)

    def delete_scan_scanner_variable(self,scan_guid,scanner_variable_guid):
        return Scans().delete_scanner_variable(scan_guid,scanner_variable_guid)

    def get_analysis_occurrences(self):
        return Occurrences().get_all()

    def get_analysis_occurrence(self,occurrence_guid):
        return Occurrences().get(guid=occurrence_guid)

    def stop_analysis_occurrence(self,occurrence_guid,save_or_delete):
        return Occurrences().stop(guid=occurrence_guid,save_or_delete=save_or_delete)

    def get_scan_occurrences(self,occurrence_guid):
        return Occurrences().get_scan_occurrences(guid=occurrence_guid)

    def get_scan_occurrence(self,scan_occ_guid):
        return ScanOccurrences().get(guid=scan_occ_guid)

    def stop_scan_occurrence(self,scan_occ_guid,save_or_delete):
        return ScanOccurrences().stop(guid=scan_occ_guid, save_or_delete=save_or_delete)

    def get_scan_occurrence_configuration(self,scan_occ_guid):
        return ScanOccurrences().get_configuration(guid=scan_occ_guid)

    def get_scan_occurrence_verification_report(self,scan_occ_guid):
        return ScanOccurrences().get_verification_report(guid=scan_occ_guid)

    def get_scan_occurrence_notes_report(self,scan_occ_guid):
        return ScanOccurrences().get_scan_notes_report(guid=scan_occ_guid)

    def get_scan_occurrence_screenshots(self,scan_occ_guid):
        return ScanOccurrences().get_screenshots(guid=scan_occ_guid)

    def get_codegroups(self):
        return CodeGroups().get_all()

    def get_codegroup(self,name):
        return CodeGroups().get(name=name)

    def get_dynamic_configuration(self):
        return Configuration().get()

    def get_dynamic_scan_capacity_summary(self):
        return ScanCapacitySummary().get()

    def get_global_scanner_variables(self):
        return ScannerVariables().get_all()

    def get_global_scanner_variable(self,guid):
        return ScannerVariables().get(guid)
    
    def create_global_scanner_variable(self,reference_key,value,description):
        return ScannerVariables().create(reference_key,value,description)

    def update_global_scanner_variable(self,guid,reference_key,value,description):
        return ScannerVariables().update(guid,reference_key,value,description)

    def delete_global_scanner_variable(self,guid):
        return ScannerVariables().delete(guid)

    def dyn_setup_user_agent(self,custom_header,type):
        return DynUtils().setup_user_agent(custom_header,type)

    def dyn_setup_custom_host(self,host_name,ip_address):
        return DynUtils().setup_custom_host(host_name,ip_address)

    def dyn_setup_blocklist(self, urls:List):
        return DynUtils().setup_blocklist(urls)

    def dyn_setup_url(self,url,directory_restriction_type='DIRECTORY_AND_SUBDIRECTORY',http_and_https=True):
        return DynUtils().setup_url(url,directory_restriction_type,http_and_https)

    def dyn_setup_scan_setting(self,blocklist_configs:list,custom_hosts:List, user_agent:None):
        return DynUtils().setup_scan_setting(blocklist_configs,custom_hosts,user_agent)

    def dyn_setup_scan_contact_info(self,email,first_and_last_name,telephone):
        return DynUtils().setup_scan_contact_info(email,first_and_last_name,telephone)

    def dyn_setup_crawl_script(self,script_body,script_type='SELENIUM'):
        return DynUtils().setup_crawl_script(script_body,script_type)

    def dyn_setup_crawl_configuration(self,scripts:List,disabled=False):
        return DynUtils().setup_crawl_configuration(scripts,disabled)

    def dyn_setup_login_logout_script(self,script_body,script_type='SELENIUM'):
        return DynUtils().setup_login_logout_script(script_body,script_type)

    def dyn_setup_auth(self,authtype,username,password,domain=None,base64_pkcs12=None,cert_name=None, login_script_data=None, logout_script_data=None):
        return DynUtils().setup_auth(authtype,username,password,domain,base64_pkcs12,cert_name,login_script_data,logout_script_data)

    def dyn_setup_auth_config(self,authentication_node:dict):
        return DynUtils().setup_auth_config(authentication_node)

    def dyn_setup_scan_config_request(self, url, allowed_hosts:List, auth_config=None, crawl_config=None, scan_setting=None):
        return DynUtils().setup_scan_config_request(url,allowed_hosts,auth_config,crawl_config,scan_setting)

    def dyn_setup_scan(self, scan_config_request, scan_contact_info=None, linked_app_guid=None):
        return DynUtils().setup_scan(scan_config_request,scan_contact_info, linked_app_guid)


# Purpose:  Log utilities

import logging
import time
from datetime import datetime

class VeracodeLog():
    def setup_logging(self,debug=False):
        now = datetime.utcnow().strftime("%Y-%m-%d-%H%M%S")
        format_string = "%(asctime)s %(levelname)s %(message)s"
        datetime_format = '%Y-%m-%d %H:%M:%S %Z'

        logging.captureWarnings(True)
        logging.Formatter.converter = time.gmtime

        if debug:
            logging.basicConfig(format=format_string, datefmt=datetime_format, filename="{}-debug.log".format(now), level=logging.DEBUG)
        else:
            logging.basicConfig(format=format_string, datefmt=datetime_format, filename="{}.log".format(now), level=logging.INFO)
            requests_logger = logging.getLogger("requests")
            requests_logger.setLevel(logging.WARNING)



# Purpose:  Exceptions


class VeracodeError(Exception):
    """Raised when something goes wrong"""
    pass


class VeracodeAPIError(Exception):
    """Raised when something goes wrong with talking to the Veracode API"""
    pass



#policy.py - API class for Policy API calls

import json

from .apihelper import APIHelper

class Policies():

    def get_all (self):
        return APIHelper()._rest_paged_request("appsec/v1/policies","GET","policy_versions",{"page": 0})
    
    def get (self,guid):
        uri = "appsec/v1/policies/{}".format(guid)
        return APIHelper()._rest_request(uri,"GET")

    def delete (self,guid):
        uri = "appsec/v1/policies/{}".format(guid)
        return APIHelper()._rest_request(uri,"DELETE")

    def create (self, name, description, vendor_policy=False, finding_rules=[], scan_frequency_rules=[], grace_periods=None):
        if grace_periods == None:
            grace_periods = {}
        return self._create_or_update("CREATE",name,description,vendor_policy,finding_rules,scan_frequency_rules,grace_periods)

    def update(self,guid, name, description, vendor_policy=False, finding_rules=[], scan_frequency_rules=[], grace_periods=None):
        if grace_periods == None:
            grace_periods = {}
        return self._create_or_update("UPDATE",name,description,vendor_policy,finding_rules,scan_frequency_rules,grace_periods,guid)

    def format_finding_rule(self,rule_type,scan_types=[],rule_value=''):
        finding_rule = {}
        finding_rule['type'] = rule_type
        finding_rule['scan_types'] = scan_types
        finding_rule['value'] = rule_value
        return finding_rule

    def format_scan_frequency_rule(self,scan_type,frequency):
        scan_frequency_rule = {}
        scan_frequency_rule['scan_type'] = scan_type
        scan_frequency_rule['frequency'] = frequency
        return scan_frequency_rule

    def format_grace_periods(self, sev5, sev4, sev3, sev2, sev1, sev0, score, sca_blocklist):
        grace_periods = {}
        grace_periods["sev5_grace_period"] = sev5
        grace_periods["sev4_grace_period"] = sev4
        grace_periods["sev3_grace_period"] = sev3
        grace_periods["sev2_grace_period"] = sev2
        grace_periods["sev1_grace_period"] = sev1
        grace_periods["sev0_grace_period"] = sev0
        grace_periods["score_grace_period"] = score
        grace_periods["sca_blacklist_grace_period"] = sca_blocklist
        return grace_periods

    def _create_or_update(self, method, name, description, vendor_policy=False, finding_rules=[], scan_frequency_rules=[], grace_periods=None, guid=None):
        if grace_periods == None:
            grace_periods = {}
        if method == 'CREATE':
            uri = 'appsec/v1/policies'
            httpmethod = 'POST'
        elif method == 'UPDATE':
            uri = 'appsec/v1/policies/{}'.format(guid)
            httpmethod = 'PUT'
        else:   
            return
        
        policy_def = {"name": name, "description": description, "vendor_policy": vendor_policy}
        policy_def["finding_rules"] = finding_rules
        policy_def["scan_frequency_rules"] = scan_frequency_rules
        policy_def.update(grace_periods)

        return APIHelper()._rest_request(uri,httpmethod,body=json.dumps(policy_def))



#sca.py - API class for SCA API calls

import json
from urllib import parse

from .apihelper import APIHelper
from .constants import Constants

class Workspaces():
     sca_base_url = "srcclr/v3/workspaces"   

     def get_all(self):
          #Gets existing workspaces
          request_params = {}
          return APIHelper()._rest_paged_request(self.sca_base_url,"GET",params=request_params,element="workspaces")

     def get_by_name(self,name):
          #Does a name filter on the workspaces list. Note that this is a partial match. Only returns the first match
          name = parse.quote(name) #urlencode any spaces or special characters
          request_params = {'filter[workspace]': name}
          return APIHelper()._rest_paged_request(self.sca_base_url,"GET",params=request_params,element="workspaces")

     def create(self,name):
          #pass payload with name, return guid to workspace
          payload = json.dumps({"name": name})
          r = APIHelper()._rest_request(self.sca_base_url,"POST",body=payload,fullresponse=True)
          loc = r.headers.get('location','')
          return loc.split("/")[-1]

     def add_team(self,workspace_guid,team_id):
          return APIHelper()._rest_request(self.sca_base_url + "/{}/teams/{}".format(workspace_guid,team_id),"PUT")

     def delete(self,workspace_guid):
          return APIHelper()._rest_request(self.sca_base_url + "/{}".format(workspace_guid),"DELETE") 

     def get_teams(self):
          return APIHelper()._rest_paged_request("srcclr/v3/teams","GET","teams",{})

     def get_projects(self,workspace_guid):
          return APIHelper()._rest_paged_request(self.sca_base_url + '/{}/projects'.format(workspace_guid),"GET","projects",{})

     def get_project(self,workspace_guid,project_guid):
          uri = self.sca_base_url + '/{}/projects/{}'.format(workspace_guid,project_guid)
          return APIHelper()._rest_request(uri,"GET")

     def get_project_issues(self,workspace_guid,project_guid):
          uri = self.sca_base_url + '/{}/projects/{}/issues'.format(workspace_guid,project_guid)
          return APIHelper()._rest_paged_request(uri,"GET","issues",{})

     def get_agents(self,workspace_guid):
          return APIHelper()._rest_paged_request(self.sca_base_url + '/{}/agents'.format(workspace_guid),"GET","agents",{})

     def get_agent(self,workspace_guid,agent_guid):
          uri = self.sca_base_url + '/{}/agents/{}'.format(workspace_guid,agent_guid)
          return APIHelper()._rest_request(uri,"GET")

     def create_agent(self,workspace_guid,name,agent_type='CLI'):
          if agent_type not in Constants().AGENT_TYPE:
               raise ValueError("{} is not in the list of valid agent types ({})".format(agent_type,Constants().AGENT_TYPE))
          uri = self.sca_base_url + '/{}/agents'.format(workspace_guid)
          body = {'agent_type': agent_type, 'name': name}
          return APIHelper()._rest_request(uri,"POST",body=json.dumps(body))

     def delete_agent(self,workspace_guid,agent_guid):
          uri = self.sca_base_url + '/{}/agents/{}'.format(workspace_guid,agent_guid)
          return APIHelper()._rest_request(uri,"DELETE")

     def get_agent_tokens(self,workspace_guid,agent_guid):
          uri = self.sca_base_url + '/{}/agents/{}/tokens'.format(workspace_guid,agent_guid)
          return APIHelper()._rest_paged_request(uri, "GET" )

     def get_agent_token(self,workspace_guid,agent_guid,token_id):
          uri = self.sca_base_url + '/{}/agents/{}/tokens/{}'.format(workspace_guid,agent_guid,token_id)
          return APIHelper()._rest_paged_request(uri, "GET" )

     def regenerate_agent_token(self,workspace_guid, agent_guid):
          uri = self.sca_base_url + '/{}/agents/{}/tokens:regenerate'.format(workspace_guid,agent_guid)
          return APIHelper()._rest_request(uri,"POST")

     def revoke_agent_token(self,workspace_guid, agent_guid, token_id):
          uri = self.sca_base_url + '/{}/agents/{}/tokens/{}'.format(workspace_guid,agent_guid,token_id)
          return APIHelper()._rest_request(uri,"DELETE")

     def get_issues(self,workspace_guid):
          uri = self.sca_base_url + '/{}/issues'.format(workspace_guid)
          return APIHelper()._rest_paged_request(uri,"GET","issues",{})

     def get_issue(self,issue_id):
          uri = self.sca_base_url + '/issues/{}'.format(issue_id)
          return APIHelper()._rest_request(uri,"GET")

     def get_libraries(self,workspace_guid,unmatched):
          if unmatched:
               uri = self.sca_base_url + '/{}/libraries/unmatched'.format(workspace_guid)
          else:
               uri = self.sca_base_url + '/{}/libraries'.format(workspace_guid)
          return APIHelper()._rest_paged_request(uri,"GET",'libraries',{})

     def get_library(self,library_id):
          uri = "srcclr/v3/libraries/{}".format(library_id)
          return APIHelper()._rest_request(uri,"GET")

     def get_vulnerability(self,vulnerability_id):
          uri = "srcclr/v3/vulnerabilities/{}".format(vulnerability_id)
          return APIHelper()._rest_request(uri,"GET")

     def get_license(self,license_id):
          uri = "srcclr/v3/licenses/{}".format(license_id)
          return APIHelper()._rest_request(uri,"GET")

     def get_scan(self,scan_id):
          return APIHelper()._rest_request("srcclr/v3/scans/{}".format(scan_id),"GET")

     def get_events(self, date_gte=None, event_group=None, event_type=None):
          baseuri = "srcclr/v3/events"
          params = {}
          if event_group != None:
               if event_group not in Constants().SCA_EVENT_GROUP:
                    raise ValueError("{} is not in the valid list of SCA event groups ({})".format(event_group,Constants().SCA_EVENT_GROUP))
               params["group"]  = event_group

          if event_type != None:
               params["type"] = event_type

          if date_gte != None:
               params["date_gte"] = date_gte

          return APIHelper()._rest_paged_request(baseuri,"GET","events",params)



#collections.py - API class for Collections API calls

import json
from urllib import parse
from typing import List

from .apihelper import APIHelper

ROOT_URL = 'was/configservice/v1'

class Analyses():
   base_url = ROOT_URL + '/analyses'

   #public methods
   def get_all(self):
      request_params = {}
      return self._get_analyses(request_params)

   def get_by_name(self,analysis_name):
      params = {"name": parse.quote(analysis_name)}
      return self._get_analyses(params)

   def get_by_target_url(self,target_url):
      params = {"target_url": target_url}
      return self._get_analyses(params)

   def get_by_search_term(self,search_term):
      params = {"search_term": parse.quote(search_term)}
      return self._get_analyses(params)

   def get(self,guid):
      uri = self.base_url + "/{}".format(guid)
      return APIHelper()._rest_request(uri,"GET")

   def get_audits(self,guid):
      uri = self.base_url + "/{}/audits".format(guid)
      return APIHelper()._rest_paged_request(uri,"GET",'analysis_audits',{'page':0})

   def create_scan(self,guid):
      uri = self.base_url + '/{}/scans'.format(guid)
      payload = {} #TODO add code for all the scan values
      return APIHelper()._rest_request(uri,"POST",json.dumps(payload))

   def get_scans(self,guid):
      uri = self.base_url + "/{}/scans".format(guid)
      return APIHelper()._rest_paged_request(uri,"GET",'scans',{'page': 0})

   def create(self,name,scans,business_unit_guid=None,email=None,owner=None):
      # basic create that adds only metadata. Use Scans().setup() to create a Scans object
      return self._create_or_update(method="CREATE",name=name,scans=scans,
                  business_unit_guid=business_unit_guid,email=email,owner=owner,guid=None)

   def update(self,guid,name,scans,business_unit_guid=None,email=None,owner=None):
      return self._create_or_update(method="UPDATE",name=name,scans=scans,
                  business_unit_guid=business_unit_guid,email=email,owner=owner,guid=guid)

   def get_scanner_variables(self,guid):
      uri = self.base_url + "/{}/scanner_variables".format(guid)
      return APIHelper()._rest_paged_request(uri,"GET", 'scanner_variables', {'page': 0})

   def update_scanner_variable(self,analysis_guid,scanner_variable_guid,reference_key,value,description):
      uri = self.base_url + '/{}/scanner_variables/{}'.format(analysis_guid,scanner_variable_guid)
      body = { 'reference_key': reference_key, 'value': value, 'description': description }
      return APIHelper()._rest_request(uri,"PUT",body)

   def delete_scanner_variable(self,analysis_guid,scanner_variable_guid):
      uri = self.base_url + '/{}/scanner_variables/{}'.format(analysis_guid,scanner_variable_guid)
      return APIHelper()._rest_request(uri,'DELETE')

   def delete(self,guid):
      uri = self.base_url + "/{}".format(guid)
      return APIHelper()._rest_request(uri,"DELETE")

   #private methods

   def _get_analyses(self,params):
      return APIHelper()._rest_paged_request(self.base_url,"GET","analyses",params=params)

   def _create_or_update(self,method,name,scans,business_unit_guid=None,email=None,owner=None,guid=None):
      if method == 'CREATE':
         uri = self.base_url
         httpmethod = 'POST'
      elif method == 'UPDATE':
         uri = self.base_url + '/{}'.format(guid)
         httpmethod = 'PUT'
      else:   
         return

      payload = {"name": name, "scans": scans}

      org_info = {}
      if business_unit_guid != None:
         org_info.update({'business_unit_id': business_unit_guid })
      if email != None:
         org_info.update({'email': email})
      if owner != None:
         org_info.update({'owner': owner})
      payload.update({'org_info': org_info})
      payload.update({"visibility": {"setup_type": "SEC_LEADS_ONLY", "team_identifiers": []}})
      return APIHelper()._rest_request(uri,httpmethod,params={},body=json.dumps(payload))

class Scans():
   base_url = ROOT_URL + '/scans'

   def get(self, guid):
      uri = self.base_url + "/{}".format(guid)
      return APIHelper()._rest_request(uri,"GET")

   def get_audits(self, guid):
      uri = self.base_url + "/{}/audits".format(guid)
      return APIHelper()._rest_paged_request(uri,"GET",'scan_audits',{'page':0})

   def get_configuration(self,guid):
      uri = self.base_url + "/{}/configuration".format(guid)
      return APIHelper()._rest_request(uri,"GET")

   def delete(self,guid):
      uri = self.base_url + '/{}'.format(guid)
      return APIHelper()._rest_request(uri, 'DELETE')

   def update(self,guid,scan):
      # use DynUtils().setup_scan() to create the scans parameter
      uri = self.base_url + '/{}'.format(guid)
      return APIHelper()._rest_request(uri,'PUT',body=json.dumps(scan))

   def get_scanner_variables(self,guid):
      uri = self.base_url + "/{}/scanner_variables".format(guid)
      return APIHelper()._rest_paged_request(uri,"GET",'scanner_variables',{'page': 0})

   def update_scanner_variable(self,scan_guid,scanner_variable_guid,reference_key,value,description):
      uri = self.base_url + '/{}/scanner_variables/{}'.format(scan_guid,scanner_variable_guid)
      body = { 'reference_key': reference_key, 'value': value, 'description': description }
      return APIHelper()._rest_request(uri,"PUT",body)

   def delete_scanner_variable(self,scan_guid,scanner_variable_guid):
      uri = self.base_url + '/{}/scanner_variables/{}'.format(scan_guid,scanner_variable_guid)
      return APIHelper()._rest_request(uri,'DELETE')   

class Occurrences():
   base_url = ROOT_URL + '/analysis_occurrences'

   def get_all(self):
      return APIHelper()._rest_paged_request(self.base_url,'GET','analysis_occurrences',{'page':0})

   def get(self,guid):
      uri = self.base_url + '/{}'.format(guid)
      return APIHelper()._rest_request(uri, 'GET')

   def stop(self, guid, save_or_delete):
      actions = {'SAVE': 'STOP_SAVE', 'DELETE': 'STOP_DELETE'}
      uri = self.base_url + '/{}'.format(guid)
      params = { 'action' : actions[save_or_delete] }
      return APIHelper()._rest_request(uri, 'PUT', json.dumps(params))

   def get_scan_occurrences(self,guid):
      uri = self.base_url + '/{}/scan_occurrences'
      return APIHelper()._rest_paged_request(uri, 'GET', 'scan_occurrences',{'page':0})

class ScanOccurrences():
   base_url = ROOT_URL + '/scan_occurrences'

   def get(self, guid):
      uri = self.base_url + '/{}'.format(guid)
      return APIHelper()._rest_request(uri,'GET')

   def stop(self,guid,save_or_delete):
      actions = {'SAVE': 'STOP_SAVE', 'DELETE': 'STOP_DELETE'}
      uri = self.base_url + '/{}'.format(guid)
      params = { 'action': actions[save_or_delete]}
      return APIHelper()._rest_request(uri,'PUT', json.dumps(params))

   def get_configuration(self,guid):
      uri = self.base_url + '/{}/configuration'.format(guid)
      return APIHelper()._rest_request(uri,'GET')
   
   def get_verification_report(self,guid):
      uri = self.base_url + '/{}/verification_report'.format(guid)
      return APIHelper()._rest_request(uri,'GET')

   def get_scan_notes_report(self,guid):
      uri = self.base_url + '/{}/scan_notes_report'.format(guid)
      return APIHelper()._rest_request(uri,'GET')

   def get_scan_screenshots(self,guid):
      uri = self.base_url + '/{}/scan_screenshots'.format(guid)
      return APIHelper()._rest_request(uri,'GET')

class CodeGroups():
   base_url = ROOT_URL + '/code_groups'

   def get_all(self):
      return APIHelper()._rest_request(self.base_url,'GET')

   def get(self,name):
      uri = self.base_url + '/{}'.format(name)
      return APIHelper()._rest_request(uri,'GET')

class Configuration():
   base_url = ROOT_URL + '/configuration'

   def get(self):
      return APIHelper()._rest_request(self.base_url,'GET')

class ScannerVariables():
   base_url = ROOT_URL + '/scanner_variables'

   def get_all(self):
      return APIHelper()._rest_paged_request(self.base_url,"GET",'scanner_variables',{'page': 0})

   def create(self,reference_key,value,description):
      payload = {'reference_key':reference_key, 'value': value, 'description': description}
      return APIHelper()._rest_request(self.base_url,'POST',json.dumps(payload))

   def get(self,guid):
      uri = self.base_url + '/{}'.format(guid)
      return APIHelper()._rest_request(uri,"GET")

   def update(self,guid,reference_key,value,description):
      uri = self.base_url + '/{}'.format(guid)
      body = { 'reference_key': reference_key, 'value': value, 'description': description }
      return APIHelper()._rest_request(uri,"PUT",body)

   def delete(self,guid):
      uri = self.base_url + '/{}'.format(guid)
      return APIHelper()._rest_request(uri,'DELETE')   

class ScanCapacitySummary():

   def get(self):
      return APIHelper()._rest_request(ROOT_URL + '/scan_capacity_summary', 'GET')

class DynUtils():

   def setup_user_agent(self,custom_header,type):
      return { "custom_header": custom_header, "type": type}

   def setup_custom_host(self,host_name,ip_address):
      return { 'host_name': host_name, 'ip_address': ip_address}

   def setup_blocklist(self, urls:List):
      return { 'black_list': urls}

   def setup_url(self,url,directory_restriction_type='DIRECTORY_AND_SUBDIRECTORY',http_and_https=True):
      # use to format any URL being made
      return { 'url': url, 'directory_restriction_type': directory_restriction_type, 'http_and_https': http_and_https}

   def setup_scan_setting(self,blocklist_configs:list,custom_hosts:List, user_agent:None):
      payload = {}
      
      if len(blocklist_configs) > 0:
         payload.update({'blacklist_configuration': blocklist_configs})

      if len(custom_hosts) > 0:
         payload.update({'custom_hosts': custom_hosts})

      if user_agent != None:
         payload.update({'user_agent':user_agent})

      return { 'scan_setting': payload }

   def setup_scan_contact_info(self,email,first_and_last_name,telephone):
      return {'scan_contact_info': {'email': email, 'first_and_last_name': first_and_last_name, 'telephone': telephone}}

   def setup_crawl_script(self,script_body,script_type='SELENIUM'):
      return { 'crawl_script_data': { 'script_body': script_body, 'script_type': script_type}}

   def setup_crawl_configuration(self,scripts:List,disabled=False):
      return { 'crawl_configuration': { 'disabled': disabled, 'scripts': scripts}}

   def setup_login_logout_script(self,script_body,script_type='SELENIUM'):
      return { 'script_body': script_body, 'script_type': script_type}

   def setup_auth(self,authtype,username,password,domain=None,base64_pkcs12=None,cert_name=None, login_script_data=None, logout_script_data=None):
      payload = {}
      if authtype == 'AUTO':
         payload.update({'AUTO': {'authtype': authtype, 'username': username, 'password': password}})
      elif authtype == 'BASIC':
         payload.update({'BASIC': {'authtype': authtype, 'username': username, 'password': password, 'domain': domain}})
      elif authtype == 'CERT':
         payload.update({'CERT': {'authtype': authtype, 'password': password, 'base64_pkcs12': base64_pkcs12, 'cert_name': cert_name}})
      elif authtype == 'FORM':
         payload.update({'FORM': {'authtype': authtype, 'login_script_data': login_script_data, 'logout_script_data': logout_script_data}})
      return payload

   def setup_auth_config(self,authentication_node:dict):
      return { 'auth_configuration': { 'authentications': authentication_node}}

   def setup_scan_config_request(self, url, allowed_hosts:List, auth_config=None, crawl_config=None, scan_setting=None):
      payload = {'target_url': url }

      if len(allowed_hosts) > 0:
         payload.update({'allowed_hosts': allowed_hosts})
      if scan_setting != None:
         payload.update(scan_setting)
      if auth_config != None:
         payload.update(auth_config)
      if crawl_config != None:
         payload.update(crawl_config)
      return { 'scan_config_request': payload }

   def setup_scan(self, scan_config_request, scan_contact_info=None, linked_app_guid=None):
      payload = {}
      payload.update( scan_config_request )
      payload.update(scan_contact_info)
      if linked_app_guid != None:
         payload.update({'linked_platform_app_uuid': linked_app_guid})
      return payload


#constants.py - contains constant values for lookups

class Constants():
     # translate between annotation types in Findings API v2 and in Mitigations API (XML)
     ANNOT_TYPE = {"APPDESIGN":"appdesign",\
          "NETENV": "netenv",\
          "OSENV":"osenv",\
          "APPROVED":"accepted",\
          "REJECTED":"rejected",\
          "FP":"fp", \
          "LIBRARY":"library", \
          "ACCEPTRISK": 'acceptrisk'}

     AGENT_TYPE = [ "CLI", "MAVEN", "GRADLE", "JENKINS", "BAMBOO", "CIRCLECI", "CODESHIP", "PIPELINES", "TRAVIS", "WINDOWSCI" ]

     SCA_EVENT_GROUP = [ 'WORKSPACE', 'AGENT', 'SCAN', 'PROJECT', 'RULES']


#findings.py - API class for Findings API and related calls

import json

from .apihelper import APIHelper

class Findings():
    def get_findings(self,app,scantype='STATIC',annot='TRUE',request_params=None,sandbox=None):
        #Gets a list of  findings for app using the Veracode Findings API
        if request_params == None:
            request_params = {}
        
        if scantype in ['STATIC', 'DYNAMIC', 'MANUAL','SCA']:
            request_params['scan_type'] = scantype
        #note that scantype='ALL' will result in no scan_type parameter as in API
            
        request_params['include_annot'] = annot

        if sandbox != None:
            request_params['context'] = sandbox
        
        uri = "appsec/v2/applications/{}/findings".format(app)
        return APIHelper()._rest_paged_request(uri,"GET","findings",request_params)

    def get_static_flaw_info(self,app,issueid,sandbox=None):
        if sandbox != None:
            uri = "appsec/v2/applications/{}/findings/{}/static_flaw_info?context={}".format(app,issueid,sandbox)
        else:
            uri = "appsec/v2/applications/{}/findings/{}/static_flaw_info".format(app,issueid)

        return APIHelper()._rest_request(uri,"GET")

    def get_dynamic_flaw_info(self,app,issueid):
        uri = "appsec/v2/applications/{}/findings/{}/dynamic_flaw_info".format(app,issueid)
        return APIHelper()._rest_request(uri,"GET")

    def add_annotation(self,app,issue_list,comment,action,sandbox=None):
        #pass issue_list as a list of issue ids
        uri = "appsec/v2/applications/{}/annotations".format(app)

        if sandbox != None:
            params = {'context': sandbox}
        else:
            params = None

        annotation_def = {'comment': comment, 'action': action}

        converted_list = [str(element) for element in issue_list]
        issue_list_string = ','.join(converted_list)
        annotation_def['issue_list'] = issue_list_string 
        
        payload = json.dumps(annotation_def)
        return APIHelper()._rest_request(uri,"POST",body=payload,params=params)

class SummaryReport():
    def get_summary_report(self,app,sandbox=None):
        if sandbox != None:
            uri = "appsec/v2/applications/{}/summary_report?context={}".format(app,sandbox)
        else:
            uri = "appsec/v2/applications/{}/summary_report".format(app)

        return APIHelper()._rest_request(uri,"GET")


#applications.py - API class for Applications API calls

import json
from urllib import parse

from .apihelper import APIHelper

class Applications():    
    def get_all(self):
        return APIHelper()._rest_paged_request('appsec/v1/applications',"GET", params={}, element="applications")

    def get (self,guid=None,legacy_id=None):
        """Gets a single applications in the current customer account using the Veracode Application API."""
        if legacy_id == None:
            apps_base_uri = "appsec/v1/applications" + "/{}"
            uri = apps_base_uri.format(guid)
        else:
            apps_base_uri = "appsec/v1/applications?legacy_id={}"
            uri = apps_base_uri.format(legacy_id)

        return APIHelper()._rest_request(uri,"GET")

    def get_by_name (self,appname):
        """Gets a list of applications having a name that matches appname, using the Veracode Applications API."""
        params = {"name": parse.quote(appname)}
        return APIHelper()._rest_paged_request(uri="appsec/v1/applications",method="GET",element="applications",params=params)

    def create(self,app_name,business_criticality, business_unit=None, teams=[]):
        return self._create_or_update("CREATE",app_name,business_criticality,business_unit,teams)

    def update(self,guid,app_name,business_criticality, business_unit=None, teams=[]):
        return self._create_or_update("UPDATE",app_name,business_criticality,business_unit,teams,guid)

    def delete(self,guid):
        uri = 'appsec/v1/applications/{}'.format(guid)
        return APIHelper()._rest_request(uri,'DELETE')

    def _create_or_update(self,method,app_name,business_criticality, business_unit=None, teams=[],guid=None):
        if method == 'CREATE':
            uri = 'appsec/v1/applications'
            httpmethod = 'POST'
        elif method == 'UPDATE':
            uri = 'appsec/v1/applications/{}'.format(guid)
            httpmethod = 'PUT'
        else:   
            return

        app_def = {'name':app_name, 'business_criticality':business_criticality}

        if len(teams) > 0:
            # optionally pass a list of teams to add to the application profile
            team_list = []
            for team in teams:
                team_list.append({'guid': team})
            app_def.update({'teams': team_list})

        if business_unit != None:
            bu = {'business_unit': {'guid': business_unit}}
            app_def.update(bu)

        payload = json.dumps({"profile": app_def})
        return APIHelper()._rest_request(uri,httpmethod,body=payload)

class Sandboxes ():
    def get_all(self,guid):
        request_params = {}
        uri = 'appsec/v1/applications/{}/sandboxes'.format(guid)
        return APIHelper()._rest_paged_request(uri,'GET','sandboxes',request_params)

    def create(self, app, name, auto_recreate=False, custom_fields=[]):
        uri = 'appsec/v1/applications/{}/sandboxes'.format(app)
        sandbox_def = {'name': name, 'auto_recreate': auto_recreate}

        if len(custom_fields) > 0:
            sandbox_def.update({"custom_fields": custom_fields})

        payload = json.dumps(sandbox_def)
        return APIHelper()._rest_request(uri,'POST',body=payload)

    def update(self, app, sandbox, name, auto_recreate=False, custom_fields=[]):
        uri = 'appsec/v1/applications/{}/sandboxes/{}'.format(app,sandbox)
        sandbox_def = {'name': name, 'auto_recreate': auto_recreate}

        if len(custom_fields) > 0:
            sandbox_def.update({"custom_fields": custom_fields})

        payload = json.dumps(sandbox_def)
        return APIHelper()._rest_request(uri,'PUT',body=payload)

    def delete(self, app, sandbox):
        uri = 'appsec/v1/applications/{}/sandboxes/{}'.format(app,sandbox)
        return APIHelper()._rest_request(uri,'DELETE')

class CustomFields():
    def get_all (self):
        return APIHelper()._rest_request('appsec/v1/custom_fields','GET')



from veracode_api_py.api import VeracodeAPI
from veracode_api_py.policy import Policies
from veracode_api_py.applications import Applications, Sandboxes, CustomFields
from veracode_api_py.collections import Collections
from veracode_api_py.dynamic import Analyses, Scans, CodeGroups, Configuration, ScannerVariables, ScanCapacitySummary, Occurrences, DynUtils
from veracode_api_py.findings import Findings, SummaryReport
from veracode_api_py.healthcheck import Healthcheck
from veracode_api_py.identity import Users, Teams, BusinessUnits, APICredentials, Roles
from veracode_api_py.sca import Workspaces
from veracode_api_py.exceptions import VeracodeError, VeracodeAPIError


#identity.py - API classes for Identity API calls

import json
import logging
from urllib import parse

from .apihelper import APIHelper

logger = logging.getLogger(__name__)

class Users():
   def get_all(self):
      #Gets a list of users using the Veracode Identity API        
      request_params = {'page': 0} #initialize the page request
      return APIHelper()._rest_paged_request("api/authn/v2/users","GET","users",request_params)

   def get_self (self):
      #Gets the user info for the current user, using the Veracode Identity API
      return APIHelper()._rest_request("api/authn/v2/users/self","GET")

   def get(self,user_guid):
      #Gets an individual user provided their GUID, using the Veracode Identity API
      uri = "api/authn/v2/users/{}".format(user_guid)
      return APIHelper()._rest_request(uri,"GET")

   def get_by_name(self,username):
      #Gets all the users who match the provided email address, using the Veracode Identity API
      request_params = {'user_name': parse.quote(username)} #initialize the page request
      return APIHelper()._rest_paged_request("api/authn/v2/users","GET","users",request_params)

   def get_user_search(self,search_term=None, api_id=None, role_id=None, login_status=None, saml_user=None, team_id=None, detailed=False):
      request_params = {'detailed': detailed}
      
      if search_term != None:
         request_params['search_term'] = parse.quote(search_term)

      if api_id != None:
         request_params['api_id'] = api_id

      if role_id != None:
         request_params['role_id'] = role_id
      
      if login_status != None:
         request_params['login_status'] = login_status

      if saml_user != None:
         request_params['saml_user'] = saml_user

      if team_id != None:
         request_params['team_id'] = team_id

      return APIHelper()._rest_paged_request("api/authn/v2/users/search","GET","users",request_params)

   def create(self,email,firstname,lastname,username=None,type="HUMAN",roles=[],teams=[]):
      user_def = { "email_address": email, "first_name": firstname, "last_name": lastname }

      rolelist = []
      if len(roles) > 0:
         for role in roles:
               rolelist.append({"role_name": role})
         user_def.update({"roles":rolelist})

      if type == "API":
         user_def.update({"user_name": username})
         user_def.update({"permissions": [{"permission_name": "apiUser"}]})
         if len(roles) == 0:
            rolelist.append({"role_name": "uploadapi"})
            rolelist.append({"role_name":"apisubmitanyscan"})
      else:
         if len(roles) == 0:
            rolelist.append({"role_name":"submitter"}) 

      if username is not None:
         user_def.update({"user_name": username})
      else:
         user_def.update({"user_name": email})

      teamlist = []
      if len(teams) > 0:
         for team in teams:
               teamlist.append({"team_id": team})
         user_def.update({"teams": teamlist})

      user_def.update({"roles": rolelist})

      payload = json.dumps(user_def)
      return APIHelper()._rest_request('api/authn/v2/users','POST',body=payload)

   def update_roles(self,user_guid,roles):
      request_params = {'partial': 'TRUE',"incremental": 'TRUE'}
      uri = "api/authn/v2/users/{}".format(user_guid)

      rolelist = []
      for role in roles:
            rolelist.append({"role_name": role})

      payload = json.dumps({"roles": rolelist})
      return APIHelper()._rest_request(uri,"PUT",request_params,body=payload)

   def update(self,user_guid,changes):
      request_params = {'partial':'TRUE',"incremental": 'TRUE'}
      uri = "api/authn/v2/users/{}".format(user_guid)
      payload = json.dumps(changes)
      return APIHelper()._rest_request(uri,"PUT",request_params,body=payload)

   def update_email_address(self,user_guid,email_address,ignore_verification=False):
      request_params = {'partial':'TRUE',"incremental": 'FALSE'}
      if ignore_verification:
         request_params['adminNoVerificationEmail'] = 'TRUE'
      uri = "api/authn/v2/users/{}".format(user_guid)
      user_def = {'email_address': email_address}
      payload = json.dumps(user_def)
      return APIHelper()._rest_request(uri,"PUT",request_params,body=payload)

   def reset_password(self,user_legacy_id):
      # Sends a password reset email for the specified user
      # If user has not yet activated, re-sends activation email instead
      uri = "api/authn/v2/users/{}/resetPassword".format(user_legacy_id)
      return APIHelper()._rest_request(uri,"POST")

   def disable(self,user_guid):
      request_params = {'partial':'TRUE'}
      uri = 'api/authn/v2/users/{}'.format(user_guid)
      payload = json.dumps({'active': False})
      return APIHelper()._rest_request(uri,"PUT",request_params,payload)

   def delete(self,user_guid):
      uri = 'api/authn/v2/users/{}'.format(user_guid)
      return APIHelper()._rest_request(uri,"DELETE")

class Teams():
   def get_all(self, all_for_org=False):
      #Gets a list of teams using the Veracode Identity API       
      if all_for_org:
         request_params = {'all_for_org': True}
      else:
         request_params = {'page': 0} #initialize the page request
      return APIHelper()._rest_paged_request("api/authn/v2/teams","GET","teams",request_params)

   def create(self, team_name, business_unit=None, members=[]):        
      team_def = {'team_name': team_name}
      
      if len(members) > 0:
         # optionally pass a list of usernames to add as team members
         users = []
         for member in members:
               users.append({'user_name': member})
         team_def.update({'users': users})

      if business_unit != None:
         bu = {'bu_id': business_unit}
         team_def.update(bu)

      payload = json.dumps(team_def)
      return APIHelper()._rest_request('api/authn/v2/teams','POST',body=payload)

   def update(self, team_guid, team_name="", business_unit=None, members=[]):
      requestbody = {}
      
      if team_name != "":
         requestbody.update({"team_name": team_name})

      if business_unit != None:
         requestbody.update({"business_unit": {"bu_id": business_unit}})

      if len(members) > 0:
         users = []
         for member in members:
               users.append({"user_name": member})
         requestbody.update({"users": users})

      if requestbody == {}:
         logging.error("No update specified for team {}".format(team_guid))

      payload = json.dumps(requestbody)
      params = {"partial":True, "incremental":True}
      uri = 'api/authn/v2/teams/{}'.format(team_guid)
      return APIHelper()._rest_request(uri,'PUT',body=payload,params=params)

   def delete(self, team_guid):
      uri = 'api/authn/v2/teams/{}'.format(team_guid)
      return APIHelper()._rest_request(uri,"DELETE")

class BusinessUnits():
   base_uri = "api/authn/v2/business_units"

   def get_all(self):
      request_params = {'page': 0}
      return APIHelper()._rest_paged_request(self.base_uri,"GET","business_units",request_params)

   def get(self,guid):
      return APIHelper()._rest_request(self.base_uri + "/{}".format(guid),"GET")

   def create(self,name,teams=[]):
      payload = {"bu_name":name}

      if len(teams)>0:
         team_list = []
         for team in teams:
            team_list.append({"team_id": team})
         payload["teams"] = team_list

      return APIHelper()._rest_request(self.base_uri,"POST",body=json.dumps(payload))

   def update(self,guid,name="",teams=[]):
      payload = {}

      if name != "":
         payload["bu_name"] = name
      
      if len(teams)>0:
         team_list = []
         for team in teams:
            team_list.append({"team_id": team})
         payload["teams"] = team_list

      return APIHelper()._rest_request(self.base_uri + "/{}".format(guid),"PUT",body=json.dumps(payload),params={"partial":True, "incremental":True})

   def delete(self,guid):
      return APIHelper()._rest_request(self.base_uri + "/{}".format(guid),"DELETE")

class APICredentials():
   base_uri = "api/authn/v2/api_credentials"
   def get_self (self):
      return APIHelper()._rest_request(self.base_uri,"GET")

   def get (self, api_id):
      return APIHelper()._rest_request(self.base_uri + '/{}'.format(api_id),"GET")

   def renew (self):
      return APIHelper()._rest_request(self.base_uri,"POST",body=json.dumps({}))

   def revoke (self, api_id):
      return APIHelper()._rest_request(self.base_uri + '/{}'.format(api_id), "DELETE")

class Roles():
   def get_all(self):
      return APIHelper()._rest_paged_request("api/authn/v2/roles","GET","roles",{'page':0})


from distutils.core import setup

setup(
  name = 'veracode_api_py',         
  packages = ['veracode_api_py'],   
  version = '0.9.14',      
  license='MIT',        
  description = 'Python helper library for working with the Veracode APIs. Handles retries, pagination, and other features of the modern Veracode REST APIs.',   
  author = 'Tim Jarrett',                  
  author_email = 'tjarrett@veracode.com',      
  url = 'https://github.com/tjarrettveracode',   
  download_url = 'https://github.com/tjarrettveracode/veracode-api-py/archive/v_094.tar.gz',    
  keywords = ['veracode', 'veracode-api'],   
  install_requires=[            
          'veracode-api-signing'
      ],
  classifiers=[
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3'
  ],
)


#healthcheck.py - API class for Healthcheck API calls

from .apihelper import APIHelper

class Healthcheck():

     def healthcheck(self):
        uri = 'healthcheck/status'
        return APIHelper()._rest_request(uri,"GET")

     def status(self):
        uri = 'https://api.status.veracode.com/status'
        return APIHelper()._rest_request(uri,"GET",use_base_url=False)


#collections.py - API class for Collections API calls

import json
from urllib import parse

from .apihelper import APIHelper

class Collections():
   compliance_titles = {
      'DID_NOT_PASS': 'Did Not Pass',
      'PASSED': 'Passed',
      'CONDITIONAL_PASS': 'Conditional Pass',
      'NOT_ASSESSED': 'Not Assessed',
      'NOT_EVALUATED': 'Not Evaluated',
      'out_of_compliance': 'Out of Compliance',
      'within_grace_period': 'Within Grace Period',
      'compliant': 'Compliant'
   }

   #public methods
   def get_all(self):
      request_params = {}
      return self._get_collections(request_params)

   def get_by_name(self,collection_name):
      params = {"name": parse.quote(collection_name)}
      return self._get_collections(params)

   def get_by_business_unit(self,business_unit_name):
      params = {"business_unit": parse.quote(business_unit_name)}
      return self._get_collections(params)

   def get_statistics(self):
      return APIHelper()._rest_request("appsec/v1/collections/statistics","GET")

   def get(self,guid):
      uri = "appsec/v1/collections/{}".format(guid)
      return APIHelper()._rest_request(uri,"GET")

   def get_assets(self,guid):
      uri = "appsec/v1/collections/{}/assets".format(guid)
      return APIHelper()._rest_paged_request(uri,"GET","assets",params={})

   def create(self,name,description="",tags='',business_unit_guid=None,custom_fields=[],assets=[]):
      return self._create_or_update(method="CREATE",name=name,description=description,
                  tags=tags,business_unit_guid=business_unit_guid,custom_fields=custom_fields,assets=assets,guid=None)

   def update(self,guid,name,description="",tags="",business_unit_guid=None,custom_fields=[],assets=[]):
      return self._create_or_update(method="UPDATE",name=name,description=description,
                  tags=tags,business_unit_guid=business_unit_guid,custom_fields=custom_fields,assets=assets,guid=guid)

   def delete(self,guid):
      uri = "appsec/v1/collections/{}".format(guid)
      return APIHelper()._rest_request(uri,"DELETE")

   #private methods

   def _get_collections(self,params):
      return APIHelper()._rest_paged_request("appsec/v1/collections","GET","collections",params=params)

   def _create_or_update(self,method,name,description="",tags="",business_unit_guid=None,custom_fields=[],assets=[],guid=None):
      if method == 'CREATE':
         uri = 'appsec/v1/collections'
         httpmethod = 'POST'
      elif method == 'UPDATE':
         uri = 'appsec/v1/collections/{}'.format(guid)
         httpmethod = 'PUT'
      else:   
         return

      payload = {"name": name, "description": description}
      if tags != '':
         t = {'tags': tags}
         payload.update(t)
      if business_unit_guid != None:
         bu = {'business_unit': {'guid': business_unit_guid} }
         payload.update(bu)
      if len(custom_fields) > 0:
         cf = {'custom_fields': custom_fields}
         payload.update(cf)
      if len(assets) > 0:
         asset_list = []
         for asset in assets:
               this_asset = {'guid':asset,'type': 'APPLICATION'}
               asset_list.append(this_asset)
               al = {'asset_infos': asset_list}
               payload.update(al)
      return APIHelper()._rest_request(uri,httpmethod,params={},body=json.dumps(payload))


#xmlapi.py - API class for legacy XML API calls

from .apihelper import APIHelper
from .constants import Constants

class XMLAPI():

    baseurl = "https://analysiscenter.veracode.com/api"

    def get_app_list(self):
        """Returns all application profiles."""
        return APIHelper()._xml_request(self.baseurl + "/4.0/getapplist.do", "GET")

    def get_app_info(self, app_id):
        """Returns application profile info for a given app ID."""
        return APIHelper()._xml_request(self.baseurl + "/5.0/getappinfo.do", "GET", params={"app_id": app_id})

    def get_sandbox_list(self, app_id):
        """Returns a list of sandboxes for a given app ID"""
        return APIHelper()._xml_request(self.baseurl + "/5.0/getsandboxlist.do", "GET", params={"app_id": app_id})

    def get_build_list(self, app_id, sandbox_id=None):
        """Returns all builds for a given app ID."""
        if sandbox_id is None:
            params = {"app_id": app_id}
        else:
            params = {"app_id": app_id, "sandbox_id": sandbox_id}
        return APIHelper()._xml_request(self.baseurl + "/4.0/getbuildlist.do", "GET", params=params)
    
    def get_build_info(self, app_id, build_id=None, sandbox_id=None):
        """Returns build info for a given build ID."""
        params = {"app_id": app_id}
        if sandbox_id != None:
            params["sandbox_id"] = sandbox_id
        if build_id != None:
            params["build_id"] = build_id
        return APIHelper()._xml_request(self.baseurl + "/5.0/getbuildinfo.do", "GET", params=params)

    def get_detailed_report(self, build_id):
        """Returns a detailed report for a given build ID."""
        return APIHelper()._xml_request(self.baseurl + "/5.0/detailedreport.do", "GET", params={"build_id": build_id})

    def set_mitigation_info(self,build_id,flaw_id_list,action,comment):
        """Adds a new mitigation proposal, acceptance, rejection, or comment for a set of flaws for an application."""
        actiontype = Constants.ANNOT_TYPE.get(action, 'comment')        
        payload = {'build_id': build_id, 'flaw_id_list': flaw_id_list, 'action': actiontype, 'comment': comment}
        return APIHelper()._xml_request(self.baseurl + "/updatemitigationinfo.do", "POST", params=payload)

    def generate_archer(self,payload):
        return APIHelper()._xml_request(self.baseurl + "/3.0/generatearcherreport.do", "GET",params=payload)

    def download_archer(self, token=None):
        if token==None:
            payload = None
        else:
            payload = {'token': token}

        return APIHelper()._xml_request(self.baseurl + "/3.0/downloadarcherreport.do", "GET", params=payload)